# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ....core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from .file_request_drive import FileRequestDrive
from .file_request_folder import FileRequestFolder
from .file_request_permissions import FileRequestPermissions


class FileRequest(pydantic_v1.BaseModel):
    """
    # The File Object

    ### Description

    The `File` object is used to represent a file in the workspace. The Object typically exists under a folder or drive, if it exists.

    ### Usage Example

    Fetch from the `GET /api/filestorage/v1/files` endpoint and view their files.
    """

    name: typing.Optional[str] = pydantic_v1.Field()
    """
    The file's name.
    """

    file_url: typing.Optional[str] = pydantic_v1.Field()
    """
    The URL to access the file.
    """

    file_thumbnail_url: typing.Optional[str] = pydantic_v1.Field()
    """
    The URL that produces a thumbnail preview of the file. Typically an image.
    """

    size: typing.Optional[int] = pydantic_v1.Field()
    """
    The file's size, in bytes.
    """

    mime_type: typing.Optional[str] = pydantic_v1.Field()
    """
    The file's mime type.
    """

    description: typing.Optional[str] = pydantic_v1.Field()
    """
    The file's description.
    """

    folder: typing.Optional[FileRequestFolder] = pydantic_v1.Field()
    """
    The folder that the file belongs to.
    """

    permissions: typing.Optional[FileRequestPermissions] = pydantic_v1.Field()
    """
    The Permission object is used to represent a user's or group's access to a File or Folder. Permissions are unexpanded by default. Use the query param `expand=permissions` to see more details under `GET /files`.
    """

    drive: typing.Optional[FileRequestDrive] = pydantic_v1.Field()
    """
    The drive that the file belongs to.
    """

    integration_params: typing.Optional[typing.Dict[str, typing.Any]]
    linked_account_params: typing.Optional[typing.Dict[str, typing.Any]]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.forbid
        json_encoders = {dt.datetime: serialize_datetime}
